#!/bin/sh

# SVN : $Id: set_led.sh 16796 2016-05-23 16:19:27Z alex.n $

if [ $# -lt 2 ]; then
	echo "Usage: $0 [on|off] [power|eth1/wan|eth2/lan|zwave|error|wifi|usb1|usb2|service]"
	exit 1
fi

MIOS_CONSTANTS_FILE="$(dirname $0)/../../mios_constants.sh"
[ -f "$MIOS_CONSTANTS_FILE" ] && source "$MIOS_CONSTANTS_FILE" || echo "FILE-NOT-FOUND: mios_constants.sh"

log_to="syslog"; log_as="set_led"
[ -f "$MIOS_FUNCTIONS_FILE" ] && source "$MIOS_FUNCTIONS_FILE" || echo "FILE-NOT-FOUND: functions.sh"

export Ppid=$PPID
export PPpid=$(awk '/^PPid:/ {print $2}' /proc/$Ppid/status)
get_app_name_from_pid() {
	[ -n "$1" ] && cat /proc/$1/cmdline | tr '\0' ' '
}

log_START "set_led.sh $* from $(get_app_name_from_pid $Ppid)[$Ppid] with parent $(get_app_name_from_pid $PPpid)[$PPpid]"

if [ -f "/proc/diag/model" ]; then
	UnitModel=$(cat /proc/diag/model 2>/dev/null)
else
	UnitModel=$(awk 'BEGIN { FS="[ \t]+:[ \t]" } /machine/ { print $2 }' /proc/cpuinfo)
fi

[ -z "$UnitModel" ] && log_DIE "Empty unit model."

case $UnitModel in

	"SERCOMM NA401")
		local on=1
		local off=0
		local flash=f
		local blink=f

		local power=0
		local eth1=1
		local wan=1
		local zwave=2
		local wifi=3
		local usb2=4
		local service=5
		local usb1=6
		local eth2=7
		local lan=7

		[ -f /proc/diag/led/power ] && eval echo \$$1\$$2 > /proc/diag/led/power
		;;

	"ASUS WL-500g Premium"*)
		local on=1
		local off=0
		local flash=f
		local led=$2
		case $led in
			wifi) led=wlan  ;;
			*)    led=power ;;
		esac

		[ -f /proc/diag/led/$led ] && eval echo \$$1 > /proc/diag/led/$led
		;;

	"MiOS v1")
		# action[on|off|flash] led_name
		# action[blink] led_name time_on time_off
		# action[mios] led_name led_mode[none,link,solid,error] #specific action so the led will be driver by the ethernet driver
		# link = to be set at startup, it will blink once per sec if has link or off if no link
			# solid = stays solid, set by NetworkMonitor if the unit has internet
			# error = blinks fast, set by NetworkMonitor if the unit doesn't have internet
			# none = turns it off
		case $2 in
			power)
				local led_name="mios:blue:power"
				;;
			eth0|eth1|eth2|lan|wan)
				local led_name="mios:orange:network"
				if [ "$1" != "mios" ]; then
					echo "ERROR: Unsupported action[$1] for ethernet led"
					exit
				fi
				;;
			zwave)
				local led_name="mios:green:ready"
				;;
			system|service)
				local led_name="mios:red:error"
				;;
		esac

		local time_on=$3
		local time_off=$4
		local led_mode=$3

		led_set_attr()
		{
			# led_set_attr "led_name" "type" "value"
			[ -f "/sys/class/leds/$1/$2" ] && echo "$3" > "/sys/class/leds/$1/$2"
		}

		case $1 in
			on)
				level=255
				led_set_attr "$led_name" "trigger" "none"
				led_set_attr "$led_name" "brightness" "$level"
				;;
			off)
				level=0
				led_set_attr "$led_name" "trigger" "none"
				led_set_attr "$led_name" "brightness" "$level"
				;;
			f|flash|flasing)
				led_set_attr "$led_name" "trigger" "heartbeat"
				;;
			blink)
				[ -z "$time_on" ] && time_on=200
				[ -z "$time_off" ] && time_off=$time_on
				led_set_attr "$led_name" "trigger" "timer"
				led_set_attr "$led_name" "delay_on" "$time_on"
				led_set_attr "$led_name" "delay_off" "$time_off"
				;;
			mios)
				led_set_attr "$led_name" "trigger" "mios"
				led_set_attr "$led_name" "device_name" "eth0"
				[ -z "$led_mode" ] && led_mode=link
				led_set_attr "$led_name" "mode" "$led_mode"
				;;
		esac
		;;

	"Sercomm NA910"|"Sercomm NA900")
		# action[on|off|flash] led_name
		# action[blink] led_name time_on time_off
		# action[mios] led_name led_mode[none,link,solid,error] #specific action so the led will be driver by the ethernet driver
			# link = to be set at startup, it will blink once per sec if has link or off if no link
			# solid = stays solid, set by NetworkMonitor if the unit has internet
			# error = blinks fast, set by NetworkMonitor if the unit doesn't have internet
			# none = turns it off
		case $2 in
			power)         local led_name="na9xx:red:power"     ;;
			eth1|wan)      local led_name="na9xx:green:wan"     ;;
			wifi)          local led_name="na9xx:green:wifi"    ;;
			wifi_act)      local led_name="na9xx:red:wifi"      ;;
			service|error) local led_name="na9xx:green:service" ;;
			eth2|lan)      local led_name="na9xx:green:lan"     ;;
			zwave)         local led_name="na9xx:green:zwave"   ;;
			usb1)          local led_name="na9xx:green:usb1"    ;;
			usb2)          local led_name="na9xx:green:usb2"    ;;
			*)             exit 1                               ;;
		esac

		local time_on=$3
		local time_off=$4
		local led_mode=$3

		led_set_attr()
		{
			# led_set_attr "led_name" "type" "value"
			[ -f "/sys/class/leds/$1/$2" ] && echo "$3" > "/sys/class/leds/$1/$2"
		}

		case $1 in
			on)
				level=255
				led_set_attr "$led_name" "trigger" "none"
				led_set_attr "$led_name" "brightness" "$level"
				;;
			off)
				level=0
				led_set_attr "$led_name" "trigger" "none"
				led_set_attr "$led_name" "brightness" "$level"
				;;
			f|flash|flasing)
				led_set_attr "$led_name" "trigger" "heartbeat"
				;;
			blink)
				[ -z "$time_on" ] && time_on=200
				[ -z "$time_off" ] && time_off=$time_on
				led_set_attr "$led_name" "trigger" "timer"
				led_set_attr "$led_name" "delay_on" "$time_on"
				led_set_attr "$led_name" "delay_off" "$time_off"
				;;
			error)
				time_on=40
				time_off=40
				led_set_attr "$led_name" "trigger" "timer"
				led_set_attr "$led_name" "delay_on" "$time_on"
				led_set_attr "$led_name" "delay_off" "$time_off"
				;;
			link)
				time_on=750
				time_off=750
				led_set_attr "$led_name" "trigger" "timer"
				led_set_attr "$led_name" "delay_on" "$time_on"
				led_set_attr "$led_name" "delay_off" "$time_off"
				;;
		esac
		;;

	"MiCasaVerde VeraLite")
			# action[on|off|flash] led_name
			# action[blink] led_name time_on time_off
			# action[mios] led_name led_mode[none,link,solid,error] #specific action so the led will be driver by the ethernet driver
				# link = to be set at startup, it will blink once per sec if has link or off if no link
				# solid = stays solid, set by NetworkMonitor if the unit has internet
				# error = blinks fast, set by NetworkMonitor if the unit doesn't have internet
				# none = turns it off
			# to not brick the mios trigger for wan led convert actions automatically
			# on -> mios solid
			# off -> mios none
			# blink/error -> mios error

		local led_state=$1
		local led_desc=$2

		local led_mode=$3
		local time_on=$3
		local time_off=$4

		case $led_desc in
			power)
				local led_name="veralite:blue:power"
				;;
			eth0|eth1|eth2|lan|wan)
				local led_name="veralite:yellow:lan"
				if [ "$led_state" != "mios" ]; then
					echo "WARNING: Unsupported action[$1] for ethernet led"
					# rewrite
					case $led_state in
						on)
							led_mode=solid
							led_state=mios
							;;
						off)
							led_mode=none
							led_state=mios
							;;
					esac
				fi
				;;
			zwave)
				local led_name="veralite:orange:zwave"
				;;
			error)
				local led_name="veralite:red:error"
				;;
			*)
				exit 1
				;;
		esac

		led_set_attr()
		{
			# led_set_attr "led_name" "type" "value"
			[ -f "/sys/class/leds/$1/$2" ] && echo "$3" > "/sys/class/leds/$1/$2"
		}

		case $led_state in
			on)
				level=255
				led_set_attr "$led_name" "trigger" "none"
				led_set_attr "$led_name" "brightness" "$level"
				;;
			off)
				level=0
				led_set_attr "$led_name" "trigger" "none"
				led_set_attr "$led_name" "brightness" "$level"
				;;
			f|flash|flasing)
				led_set_attr "$led_name" "trigger" "heartbeat"
				;;
			blink)
				[ -z "$time_on" ] && time_on=200
				[ -z "$time_off" ] && time_off=$time_on
				led_set_attr "$led_name" "trigger" "timer"
				led_set_attr "$led_name" "delay_on" "$time_on"
				led_set_attr "$led_name" "delay_off" "$time_off"
				;;
			mios)
				led_set_attr "$led_name" "trigger" "mios"
				led_set_attr "$led_name" "device_name" "eth0"
				[ -z "$led_mode" ] && led_mode=link
				led_set_attr "$led_name" "mode" "$led_mode"
				;;
		esac
		;;

	"Sercomm NA930")
		# action[on|off|flash] led_name
		# action[blink] led_name time_on time_off
		# action[mios] led_name led_mode[none,link,solid,error] #specific action so the led will be driver by the ethernet driver
			# link = to be set at startup, it will blink once per sec if has link or off if no link
			# solid = stays solid, set by NetworkMonitor if the unit has internet
			# error = blinks fast, set by NetworkMonitor if the unit doesn't have internet
			# none = turns it off
		# to not brick the mios trigger for wan led convert actions automatically
		# on -> mios solid
		# off -> mios none
		# blink/error -> mios error

		local led_state=$1
		local led_desc=$2

		local led_mode=$3
		local time_on=$3
		local time_off=$4

		local GCOM_SCRIPTS_PATH="/etc/gcom"

		case $led_desc in
			power)
				local led_name="na930:blue:power"
				;;
			zwave)
				local led_name="na930:blue:zwave"
				;;
			status|usb|usb1)
				local led_name="na930:blue:status"
				;;
			service|error)
				local led_name="na930:blue:service"
				;;
			gsm|cell|3g)
				local led_name="cell"
				local modemPort=$(get_3g_modem_port -type control -cache 2>/dev/null)
				if [ -z "$modemPort" ]; then
					log_ERROR "Failed to get GSM modem port"
					exit 1
				fi
				;;
			*)
				exit 1
				;;
		esac

		led_set_attr()
		{
			# led_set_attr "led_name" "type" "value"
			[ -f "/sys/class/leds/$1/$2" ] && echo "$3" > "/sys/class/leds/$1/$2"
		}

		case $led_state in
			on)
				if [ "$led_name" != "cell" ]; then
					level=255
					led_set_attr "$led_name" "trigger" "none"
					led_set_attr "$led_name" "brightness" "$level"
				else
					gcom -d $modemPort -s $GCOM_SCRIPTS_PATH/led_on.gcom
				fi
				;;
			off)
				if [ "$led_name" != "cell" ]; then
					level=0
					led_set_attr "$led_name" "trigger" "none"
					led_set_attr "$led_name" "brightness" "$level"
				else
					gcom -d $modemPort -s $GCOM_SCRIPTS_PATH/led_off.gcom
				fi
				;;
			blink)
				[ -z "$time_on" ] && time_on=200
				[ -z "$time_off" ] && time_off=$time_on
				if [ "$led_name" != "cell" ]; then
					led_set_attr "$led_name" "trigger" "timer"
					led_set_attr "$led_name" "delay_on" "$time_on"
					led_set_attr "$led_name" "delay_off" "$time_off"
				else
					time_on=$((time_on / 100))
					[ $time_on -eq 0 ] && time_on=1
					time_off=$((time_off / 100))
					[ $time_off -eq 0 ] && time_off=1
					# led_blink.gcom uses the ON and OFF environment variables.
					export ON=$time_on
					export OFF=$time_on
					gcom -d $modemPort -s $GCOM_SCRIPTS_PATH/led_blink.gcom
				fi
				;;
			mios)
				led_set_attr "$led_name" "trigger" "mios"
				led_set_attr "$led_name" "device_name" "eth0"
				[ -z "$led_mode" ] && led_mode=link
				led_set_attr "$led_name" "mode" "$led_mode"
				;;
		esac
		;;

	"Sercomm NA301")
		# action[on|off|flash] led_name
		# action[blink] led_name time_on time_off
		# action[mios] led_name led_mode[none,link,solid,error] #specific action so the led will be driver by the ethernet driver
			# link = to be set at startup, it will blink once per sec if has link or off if no link
			# solid = stays solid, set by NetworkMonitor if the unit has internet
			# error = blinks fast, set by NetworkMonitor if the unit doesn't have internet
			# none = turns it off
		# to not brick the mios trigger for wan led convert actions automatically
		# on -> mios solid
		# off -> mios none
		# blink/error -> mios error

		local led_state=$1
		local led_desc=$2

		local led_mode=$3
		local time_on=$3
		local time_off=$4

		case $led_desc in
			power)
				local led_name="na301:green:power"
				;;
			zwave)
				local led_name="na301:green:zwave"
				;;
			lan|wan)
				local led_name="na301:green:lan"
				;;
			service|error)
				local led_name="na301:green:service"
				;;
			*)
				exit 1
				;;
		esac

		led_set_attr()
		{
			# led_set_attr "led_name" "type" "value"
			[ -f "/sys/class/leds/$1/$2" ] && echo "$3" > "/sys/class/leds/$1/$2"
		}

		case $led_state in
			on)
				level=255
				led_set_attr "$led_name" "trigger" "none"
				led_set_attr "$led_name" "brightness" "$level"
				;;
			off)
				level=0
				led_set_attr "$led_name" "trigger" "none"
				led_set_attr "$led_name" "brightness" "$level"
				;;
			blink)
				[ -z "$time_on" ] && time_on=200
				[ -z "$time_off" ] && time_off=$time_on
				led_set_attr "$led_name" "trigger" "timer"
				led_set_attr "$led_name" "delay_on" "$time_on"
				led_set_attr "$led_name" "delay_off" "$time_off"
				;;
			mios|link)
				led_set_attr "$led_name" "trigger" "netdev"
				led_set_attr "$led_name" "device_name" "eth0"
				[ -z "$led_mode" ] && led_mode="link rx tx"
				led_set_attr "$led_name" "mode" "$led_mode"
				;;
		esac
		;;

	"Sercomm G450")
		# Usage:
		# set_led.sh on|off led_name
		# set_led.sh blink led_name time_on time_off

	  ledState=$1
		ledName=$2
		timeOn=$3
		timeOff=$4

		ledGpioId=0

		case $ledName in
			power)      ledGpioId=1  ;;
			internet)   ledGpioId=2  ;;
			wifi)    	ledGpioId=3  ;;
			zwave)      ledGpioId=4  ;;
			service)    ledGpioId=5  ;;
			zigbee)     ledGpioId=6  ;;
			bluetooth)  ledGpioId=7  ;;
			all)        ledGpioId=99 ;;
			*)          exit 1 ;;
		esac

		led_set_attr()
		{
			# Usage: led_set_attr  state led_gpio_id [time_on] [time_off]
			# echo ledon      <id>       > /proc/sercomm_gpio
			# echo ledoff     <id>       > /proc/sercomm_gpio
			# echo ledblink   <id> <on_delay*100ms> <off_delay*100ms> > /proc/sercomm_gpio
			# echo ledinfo               > /proc/sercomm_gpio
			# echo ledonall              > /proc/sercomm_gpio
			# echo ledoffall             > /proc/sercomm_gpio
			[ -f "/proc/sercomm_gpio" ] && echo "$1" "$2" $3 $4 > "/proc/sercomm_gpio" # 3 and 4 are optional, so don't quote them
		}

		LED_GPIO_IDS='1 2 3 4 5 6 7'

		case $ledState in
			on)
				if [ $ledGpioId != 99 ]; then
					led_set_attr "ledon" $ledGpioId
				else
					led_set_attr "ledonall"
				fi
				;;
			off)
				if [ $ledGpioId != 99 ]; then
					led_set_attr "ledoff" $ledGpioId
				else
					led_set_attr "ledoffall"
				fi
				;;
			blink)
				timeOn=$((${timeOn:-200} / 100))
				timeOff=$((${timeOff:-$((timeOn*100))} / 100))
				# If timeOn or timeOff are 0, it will automatically default to 100ms
				if [ $ledGpioId != 99 ]; then
					led_set_attr "ledblink" $ledGpioId $timeOn $timeOff
				else
					for ledGpioId in $LED_GPIO_IDS; do
						led_set_attr "ledblink" $ledGpioId $timeOn $timeOff
					done
				fi
				;;
			*)
				echo "Not implemented" >&2
				exit 1
				;;
		esac
		;;


	"Samsung CY-SWR1100")
		:
		;;

esac
